#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from unittest import mock

import testtools

from magnumclient.osc import plugin


class TestMakeClient(testtools.TestCase):

    def _make_instance(self, api_version):
        instance = mock.Mock()
        instance._api_version = {'container_infra': api_version}
        return instance

    def _call_make_client(self, api_version):
        instance = self._make_instance(api_version)
        with mock.patch('osc_lib.utils.get_client_class') as mock_gcc:
            mock_client_class = mock.Mock(return_value=mock.Mock())
            mock_gcc.return_value = mock_client_class
            plugin.make_client(instance)
            return mock_gcc, mock_client_class

    def test_major_version_uses_latest(self):
        """Bare major version '1' results in api_version='latest'."""
        mock_gcc, mock_client_class = self._call_make_client('1')
        mock_gcc.assert_called_once_with('container_infra', '1', mock.ANY)
        _, kwargs = mock_client_class.call_args
        self.assertEqual('latest', kwargs['api_version'])

    def test_microversion_forwarded(self):
        """Full microversion '1.12' is forwarded to the HTTP client."""
        mock_gcc, mock_client_class = self._call_make_client('1.12')
        # Major part must be used for the class lookup
        mock_gcc.assert_called_once_with('container_infra', '1', mock.ANY)
        # Full microversion must reach the HTTP client
        _, kwargs = mock_client_class.call_args
        self.assertEqual('1.12', kwargs['api_version'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from magnumclient.v1 import basemodels


CLUSTER_TEMPLATE_ATTRIBUTES = [
    'insecure_registry',
    'labels',
    'updated_at',
    'floating_ip_enabled',
    'fixed_subnet',
    'master_flavor_id',
    'uuid',
    'no_proxy',
    'https_proxy',
    'tls_disabled',
    'keypair_id',
    'public',
    'http_proxy',
    'docker_volume_size',
    'server_type',
    'external_network_id',
    'cluster_distro',
    'image_id',
    'volume_driver',
    'registry_enabled',
    'docker_storage_driver',
    'apiserver_port',
    'name',
    'created_at',
    'network_driver',
    'fixed_network',
    'coe',
    'flavor_id',
    'master_lb_enabled',
    'dns_nameserver',
    'project_id',
    'hidden',
    'tags',
    'driver',
]

CREATION_ATTRIBUTES = basemodels.CREATION_ATTRIBUTES
CREATION_ATTRIBUTES.append('insecure_registry')
CREATION_ATTRIBUTES.append('driver')


class ClusterTemplate(basemodels.BaseModel):
    model_name = "ClusterTemplate"


class ClusterTemplateManager(basemodels.BaseModelManager):
    api_name = "clustertemplates"
    resource_class = ClusterTemplate



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from magnumclient.v1 import baseunit


CLUSTER_ATTRIBUTES = [
    'status',
    'health_status',
    'cluster_template_id',
    'node_addresses',
    'uuid',
    'cluster_id',
    'status_reason',
    'created_at',
    'updated_at',
    'coe_version',
    'labels',
    'labels_overridden',
    'labels_skipped',
    'labels_added',
    'fixed_network',
    'fixed_subnet',
    'floating_ip_enabled',
    'faults',
    'keypair',
    'api_address',
    'master_addresses',
    'master_lb_enabled',
    'create_timeout',
    'node_count',
    'discovery_url',
    'docker_volume_size',
    'master_count',
    'container_version',
    'name',
    'master_flavor_id',
    'flavor_id',
    'health_status_reason',
    'project_id',
]

CREATION_ATTRIBUTES = baseunit.CREATION_ATTRIBUTES
CREATION_ATTRIBUTES.append('cluster_template_id')
CREATION_ATTRIBUTES.append('create_timeout')
CREATION_ATTRIBUTES.append('keypair')
CREATION_ATTRIBUTES.append('docker_volume_size')
CREATION_ATTRIBUTES.append('labels')
CREATION_ATTRIBUTES.append('master_flavor_id')
CREATION_ATTRIBUTES.append('flavor_id')
CREATION_ATTRIBUTES.append('fixed_network')
CREATION_ATTRIBUTES.append('fixed_subnet')
CREATION_ATTRIBUTES.append('floating_ip_enabled')
CREATION_ATTRIBUTES.append('merge_labels')
CREATION_ATTRIBUTES.append('master_lb_enabled')


class Cluster(baseunit.BaseTemplate):
    template_name = "Clusters"


class ClusterManager(baseunit.BaseTemplateManager):
    resource_class = Cluster
    template_name = 'clusters'

    @staticmethod
    def _normalize(cluster):
        # Pre-1.13 servers return stack_id instead of cluster_id; normalise
        # so callers always see cluster_id regardless of server microversion.
        stack_id = cluster._info.get('stack_id')
        if stack_id and not cluster._info.get('cluster_id'):
            cluster._info['cluster_id'] = stack_id
        return cluster

    def get(self, id):
        cluster = super().get(id)
        return self._normalize(cluster) if cluster else None

    def list(self, **kwargs):
        return [self._normalize(c) for c in super().list(**kwargs)]

    def resize(self, cluster_uuid, node_count,
               nodes_to_remove=[], nodegroup=None):
        url = self._path(cluster_uuid) + "/actions/resize"

        post_body = {"node_count": node_count}
        if nodes_to_remove:
            post_body.update({"nodes_to_remove": nodes_to_remove})
        if nodegroup:
            post_body.update({"nodegroup": nodegroup})

        resp, resp_body = self.api.json_request("POST", url, body=post_body)

        if resp_body:
            return self.resource_class(self, resp_body)

    def upgrade(self, cluster_uuid, cluster_template,
                max_batch_size=1, nodegroup=None):
        url = self._path(cluster_uuid) + "/actions/upgrade"

        post_body = {"cluster_template": cluster_template}
        if max_batch_size:
            post_body.update({"max_batch_size": max_batch_size})
        if nodegroup:
            post_body.update({"nodegroup": nodegroup})

        resp, resp_body = self.api.json_request("POST", url, body=post_body)

        if resp_body:
            return self.resource_class(self, resp_body)
